<?php
class Admin_Kb_Controller extends Admin_Base {
	public function get_index()
	{
		$posts = Kb::order_by('updated_at', 'desc')
			->get();

		Title::put('Knowledge Base Admin');
		$this->layout->nest('content', 'admin.kb.index', array('posts' => $posts));
	}

	public function get_new()
	{
		Asset::add('lepture-css', 'css/editor.css');
		Asset::add('lepture-js', 'js/editor.js');

		Title::put('New Knowledge Base Article');

		$this->layout->nest('content', 'admin.kb.new');
	}

	public function post_new()
	{
		$rules = array(
			'title' => 'required|min:3|max:200',
			'message' => 'required|min:1',
		);
		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors)
				->with_input();

		$slug = Str::slug($input['title']);

		// Ensure the slug isn't taken
		$find_slug = Kb::where_slug($slug)
			->count();

		// Append a random string if it's taken
		if ($find_slug > 0)
			$slug = $find_slug.Str::random(4);

		$post = new Kb;
		$post->title = $input['title'];
		$post->content = $input['message'];
		$post->slug = $slug;
		$post->category = '';
		$post->tags = '';
		$post->save();

		return Redirect::to('admin/kb/edit/'.$post->id)
			->with('good', 'Post created.');

	}

	public function get_edit($id = -1)
	{
		$post = Kb::find($id);
		if ($post == NULL) return Response::error(404);

		Title::put('Editing Knowledge Base');
		
		Asset::add('lepture-css', 'css/editor.css');
		Asset::add('lepture-js', 'js/editor.js');

		$this->layout->nest('content', 'admin.kb.edit', array('post' => $post));
	}

	public function post_edit($id = -1)
	{
		$post = Kb::find($id);
		if ($post == NULL) return Response::error(404);

		$rules = array(
			'title' => 'required|min:3|max:200',
			'message' => 'required|min:1',
		);
		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors)
				->with_input();

		$slug = Str::slug($input['title']);

		// Ensure the slug isn't taken
		$find_slug = Kb::where_slug($slug)
			->where('id', '!=', $post->id)
			->count();

		// Append a random string if it's taken
		if ($find_slug > 0)
			$slug = $find_slug.Str::random(4);

		$post->title = $input['title'];
		$post->content = $input['message'];
		$post->slug = $slug;
		$post->category = '';
		$post->tags = '';
		$post->save();

		return Redirect::to('admin/kb/edit/'.$post->id)
			->with('good', 'Post updated.');
	}

	public function get_view($string = '')
	{
		if (empty($string))
			return Response::error(404);

		$find = Kb::where_slug($string)->first();
		if ($find == NULL) return Response::error(404);
		
	}
}