<?php
class Admin_Ticket_Controller extends Admin_Base {    
	public function get_index()
	{
		Title::put('Tickets');

		$status = (isset($_GET['status']) AND $_GET['status'] !== '') ? $_GET['status'] : NULL;
		$priority = (isset($_GET['priority']) AND $_GET['priority'] !== '') ? $_GET['priority'] : NULL;
		$tickets_q = Ticket::order_by('updated_at', 'desc');

		if ($status !== NULL)
			$tickets_q->where_status($status);

		if ($priority !== NULL)
			$tickets_q->where_priority($priority);
		
		$this->layout->nest('content', 'admin.ticket.index', array(
			'tickets' => $tickets_q->get(),
			'status' => $status,
			'priority' => $priority
		));
	}

	public function get_assigned()
	{
		Title::put('Assigned Tickets');
		$status = (isset($_GET['status']) AND $_GET['status'] !== '') ? $_GET['status'] : NULL;
		$priority = (isset($_GET['priority']) AND $_GET['priority'] !== '') ? $_GET['priority'] : NULL;
		$tickets_q = Ticket::order_by('updated_at', 'desc');

		if ($status !== NULL)
			$tickets_q->where_status($status);

		if ($priority !== NULL)
			$tickets_q->where_priority($priority);
		
		$tickets_q->where_assigned(Auth::user()->id);
		
		$this->layout->nest('content', 'admin.ticket.index', array(
			'tickets' => $tickets_q->get(),
			'status' => $status,
			'priority' => $priority
		));
	}

	public function get_view($ticket_id, $arguments = array())
	{
		if (! is_numeric($ticket_id) OR ! Auth::check())
			return Response::error(404);

		$ticket = Ticket::find($ticket_id);

		if ($ticket == NULL)
			return Response::error(404);

		Title::put('Ticket #'.$ticket->id);
		
		Asset::add('tag-manager', 'js/tagmanager.js');

		Asset::add('lepture-css', 'css/editor.css');
		Asset::add('lepture-js', 'js/editor.js');
		
		$this->layout->nest('content', 'admin.ticket.view', array(
			'ticket' => $ticket,
			'responses' => $ticket->responses()->order_by('created_at', 'asc')->get(),
		));
	}

	public function post_view($ticket_id)
	{
		if (! is_numeric($ticket_id) OR ! Auth::check())
			return Response::error(404);

		$ticket = Ticket::find($ticket_id);

		if ($ticket == NULL)
			return Response::error(404);

		if ((int) $ticket->user_id !== (int) Auth::user()->id OR $ticket->status !== 'open')
			return Response::error(404);

		if (isset($_POST['action']) AND $_POST['action'] == 'update-meta') :
			$rules = array(
				'priority' => 'in:low,normal,high'
			);

			$input = Input::all();
			$validator = Validator::make(Input::all(), $rules);
			
			if ($validator->fails())
				return Redirect::back()
					->with_input()
					->with('error', $validator->errors);

			// Assigning it to somebody?
			if ($input['assign-to'] !== 'nobody' AND $ticket->assigned !== $input['assign-to']) :
				$user = User::find($input['assign-to']);

				$link = URL::to('ticket/'.$ticket->id);
				$email_message = <<<EOM
<p>Hi!</p>

<p>You have been assigned ticket #{$ticket->id} <strong>{$ticket->subject}</stong>.
You can view the ticket at <a href="$link">here</a> or by clicking the link below.</p>

<p><a href="$link">$link</a></p>
EOM;
				$body = View::make('global.email')
					->with('subject', 'Assigned to Ticket "'.$ticket->subject.'"')
					->with('message', $email_message);

				$mailer = IoC::resolve('mailer');
				$message = Swift_Message::newInstance('Assigned to Ticket "'.$ticket->subject.'"')
				    ->setFrom(array(Config::get('engine.site-email') => Config::get('engine.site-name')))
				    ->setTo(array($user->email => $user->name))
				    ->setBody($body,'text/html');

				// Send the email
				$mailer->send($message);
			endif;

			$ticket->priority = $input['priority'];
			$ticket->assigned = ($input['assign-to'] == 'nobody') ? NULL : $input['assign-to'];
			$ticket->save();

			// Remove old tags
			Tag::where_ticket_id($ticket->id)->delete();

			if (isset($input['hidden-tagsjax'])) :
				$tags = explode(',', $input['hidden-tagsjax']);

				foreach($tags as $tag) :
					$tag = strtolower($tag);
					if (Tag::where_tag($tag)->where_ticket_id($ticket->id)->count() > 0)
						continue;

					$o = new Tag;
					$o->tag = $tag;
					$o->ticket_id = $ticket->id;
					$o->save();
				endforeach;
			endif;

			return Redirect::back()
				->with('good', 'Updated!');
		endif;

		$rules = array(
			'message' => 'required|min:10',
		);

		$input = Input::all();
		$validator = Validator::make(Input::all(), $rules);
		
		if ($validator->fails())
			return Redirect::back()
				->with_input()
				->with('error', $validator->errors);

		// Actions on the ticket
		$actions = array();

		$resp = new Ticket_Response;
		$resp->is_staff = 1;
		$resp->ticket_id = $ticket->id;
		$resp->user_id = Auth::user()->id;
		$resp->contents = $input['message'];

		if (count($actions) >0)
			$resp->actions = serialize($actions);

		$resp->save();

		// Update the ticket itself
		$carbon = new Carbon\Carbon;

		// Are we closing?
		if ($input['submit-item'] == 'update-close')
			$ticket->status = 'closed';

		$ticket->updated_at = $carbon::now()->toDateTimeString();
		$ticket->save();

		// Send an email to the user
		$mailer = IoC::resolve('mailer');

		// Construct the message
		$link = URL::to('ticket/'.$ticket->id);
		$user = User::find($ticket->user_id);
		$email_message = <<<EOM
<p>Hi!</p>

<p>An update has been posted to ticket #{$ticket->id} <strong>{$ticket->subject}</stong>.
You can view the update <a href="$link">here</a> or by clicking the link below.</p>

<p><a href="$link">$link</a></p>
EOM;
		$body = View::make('global.email')
			->with('subject', 'New Response to "'.$ticket->subject.'"')
			->with('message', $email_message);

		$message = Swift_Message::newInstance('New Response to "'.$ticket->subject.'"')
		    ->setFrom(array(Config::get('engine.site-email') => Config::get('engine.site-name')))
		    ->setTo(array($user->email => $user->name))
		    ->setBody($body,'text/html');
		    // ->addPart('My Plain Text Message','text/plain')

		// Send the email
		$mailer->send($message);
		
		return Redirect::to('admin/ticket/'.$ticket->id)
			->with('good', 'Ticket updated.');
	}

	/**
	 * Query some tags for a ticket
	 */
	public function get_tags()
	{
		Config::set('application.profiler', false);
		$tags = DB::table('tags')
			->distinct()
			->select('tag')
			->get();
		
		$object = new stdClass;
		$object->tags = $tags;
		return Response::json($object);
	}

	public function post_tags_update($ticket_id)
	{
		if (! is_numeric($ticket_id) OR ! Auth::check())
			return Response::error(404);

		$ticket = Ticket::find($ticket_id);

		if ($ticket == NULL)
			return Response::error(404);

		if ((int) $ticket->user_id !== (int) Auth::user()->id OR $ticket->status !== 'open')
			return Response::error(404);

		if (! isset($_POST['tag']))
			return Response::make('ok');

		$tag = strtolower(trim($_POST['tag']));

		$has_tag = Tag::where_ticket_id($ticket_id)
			->where_tag($tag)
			->count();

		if ($has_tag > 0)
			return Response::make('ok');

		$t = new Tag;
		$t->ticket_id = $ticket_id;
		$t->tag = $tag;
		$t->save();
		return Response::make('ok');
	}
}