<?php

class Admin_Users_Controller extends Admin_Base {    
	public function get_index()
	{
		Title::put('Users');
		$this->layout->nest('content', 'admin.users.index', array(
			'users' => User::get()
		));
	}

	public function get_new()
	{
		Title::put('New User');
		$this->layout->nest('content', 'admin.users.new');
	}

	public function post_new()
	{
		$rules = array(
			'name' => 'required|min:1',
			'role' => 'required|in:user,staff',
			'email' => 'required|email|unique:users,email',
			'password' => 'min:5',
		);

		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors);

		if (empty($input['password'])) :
			$generated = TRUE;
			$input['password'] = Str::random(8);
		else :
			$generated = FALSE;
		endif;

		$user = new User;
		$user->email = $input['email'];
		$user->name = $input['name'];
		$user->type = $input['role'];
		$user->password = Hash::make($input['password']);

		$user->save();

		// Send them an email
		$mailer = IoC::resolve('mailer');

		// Construct the message
		$link = URL::base();
		$site_name = Config::get('engine.site-name');
		$password = ($generated) ? $input['password'] : '**omitted**';

		$email_message = <<<EOM
<p>Hi!</p>

<p>A new account for you has been made at <strong>$site_name</strong>. You can login <a href="$link">here</a> with the following credentials:</p>

<p><strong>Email Address:</strong> {$user->email}<br />
<strong>Password:</strong> $password</p>

<p>Thanks!</p>
EOM;
		$body = View::make('global.email')
			->with('subject', 'New Account at '.Config::get('engine.site-name'))
			->with('message', $email_message);

		$message = Swift_Message::newInstance('New Account at '.Config::get('engine.site-name'))
		    ->setFrom(array(Config::get('engine.site-email') => Config::get('engine.site-name')))
		    ->setTo(array($user->email => $user->name))
		    ->setBody($body,'text/html');
		    // ->addPart('My Plain Text Message','text/plain')

		// Send the email
		$mailer->send($message);

		return Redirect::to('admin/users/view/'.$user->id)
			->with('good', 'User created.');
	}

	public function get_view($id = -1)
	{
		$find = User::find($id);
		if ($find == NULL) return Response::error(404);

		if ($find->type == 'user')
			$tickets = Ticket::where_user_id($find->id)
				->order_by('updated_at', 'DESC')
				->get();
		elseif ($find->type == 'staff')
			$tickets = Ticket::where_assigned($find->id)
				->order_by('updated_at', 'DESC')
				->get();
		else
			$tickets = array();

		Title::put($find->name);
		$this->layout->nest('content', 'admin.users.view', array(
			'user' => $find,
			'tickets' => $tickets,
		));
	}

	public function get_delete($id = -1)
	{
		$find = User::find($id);
		if ($find == NULL) return Response::error(404);

		if ($find->id == Auth::user()->id)
			return Response::error(404);

		Title::put('Delete '.$find->name);
		$this->layout->nest('content', 'admin.users.delete', array(
			'user' => $find,
		));
	}

	public function post_delete($id = -1)
	{
		$find = User::find($id);
		if ($find == NULL) return Response::error(404);

		if ($find->id == Auth::user()->id)
			return Response::error(404);

		$find->delete();
		return Redirect::to('admin/users')
			->with('good', 'User deleted.');
	}

	public function post_view($id = -1)
	{
		$user = User::find($id);
		if ($user == NULL) return Response::error(404);

		$rules = array(
			'name' => 'required|min:1',
			'role' => 'required|in:user,staff',
			'email' => 'required|email|unique:users,email,'.$user->id,
		);

		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors);

		$user->email = $input['email'];
		$user->name = $input['name'];
		$user->type = $input['role'];

		$user->save();

		return Redirect::back()
			->with('good', 'User updated.');
	}
}