<?php
abstract class Base_Controller extends Controller {
    // Default is REST
    public $restful = true;
    public $layout = 'global.layout';
    public $installer = FALSE;

    public function __construct()
    {
        parent::__construct();

        // CSRF
        $this->filter('before', 'csrf')->on('post');
        
        // Load jQuery
        Asset::add('jquery', 'https://ajax.googleapis.com/ajax/libs/jquery/1.8.1/jquery.min.js');
        
        // Load the Assets
        Asset::add('foundation-css', 'css/foundation.min.css');
        Asset::add('foundation-js', 'js/foundation.min.js');

        Asset::add('master', 'css/application.css');
        Asset::add('master-js', 'js/application.js');

        // Commonly Used Things
        Form::macro('markup', function($text, $attributes = array())
        {
            return "<p ".HTML::attributes($attributes).">{$text}</p>";   
        });
        
        if (! file_exists(path('app').'/config/database.php') AND ! $this->installer)
		$this->filter('before', 'to_installer');
    }

    /**
     * Catch-all method for requests that can't be matched.
     *
     * @param  string    $method
     * @param  array     $parameters
     * @return Response
     */
    public function __call($method, $parameters)
    {
        return Response::error('404');
    }

	/**
	 * Go to Installer
	*/
	private function to_installer()
	{
		return Redirect::to('installer');
	}
}