<?php
class Frontend_Controller extends Base_Controller {
	public function get_index()
	{
		$message = Option::retrieve('index-message');
		if ($message !== NULL) :
			$cache = 'index:'.md5($message);
			if (Cache::has($cache)) :
				$message = Cache::get($cache);
			else :
				$parser = new dflydev\markdown\MarkdownExtraParser();
				$message = $parser->transformMarkdown($message);
				Cache::put($cache, $message, 60);
			endif;
		endif;

		$this->layout->nest('content', 'frontend.index', array('message' => $message));
	}

	public function get_login()
	{
		if (Auth::check())
			return Response::error(404);

		Title::put('Login');
		$this->layout->nest('content', 'user.login');
	}

	public function post_login()
	{
		if (Auth::check())
			return Response::error(404);

		$input = Input::all();
		if (! isset($input['email']) OR ! isset($input['password']))
			return Response::error(404);

		if (Auth::attempt(array(
			'username' => $input['email'],
			'password' => $input['password'],
		)))
			return Redirect::to((Auth::user()->type == 'staff') ? 'admin' : 'tickets')
				->with('good', 'Welcome back!');
		else
			return Redirect::back()
				->with('error', 'We cannot find a user with that matches that email/password.');
	}

	public function get_account()
	{
		if (! Auth::check())
			return Response::error(404);

		Title::put('Account');

		$this->layout->nest('content', 'user.account', array(
			'user' => Auth::user()
		));
	}

	public function post_account()
	{
		if (! Auth::check())
			return Response::error(404);

		$rules = array(
			'email' => 'required|email|unique:users,email,'.Auth::user()->id,
			'password' => 'min:5',
		);

		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors);

		$user = Auth::user();
		$user->email = $input['email'];

		if (! empty($input['password']))
			$user->password = Hash::make($input['password']);

		$user->save();

		return Redirect::back()
			->with('good', 'Account updated.');
	}
}