<?php
class Installer_Controller extends Base_Controller
{
	public $layout = 'installer.layout';
	public $installer = TRUE;
	
	public function __construct()
	{
		parent::__construct();
	}

	public function get_index()
	{
		if (Engine::$installed)
			return Redirect::to('installer/settings');

		Title::put('Installer');
		$this->layout->nest('content', 'installer.index');
	}

	public function post_index()
	{
		if (Engine::$installed)
			return Redirect::to('installer/settings');
		
		$rules = array(
			'db-host' => 'required',
			'db-user' => 'required',
			'db-name' => 'required',
			'db-password' => 'required',
		);
		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors);

		// Let's test the connection
		try {
			$dsn = "mysql:host={$input['db-host']};dbname={$input['db-name']}";
			$connection = new PDO($dsn, $input['db-user'], $input['db-password']);
		} catch (PDOException $e)
		{
			die($e->getMessage());
		}

		// Generate the config
		$file = File::get(path('app').'/config/database-tmp.php');
		$file = str_replace('DB_HOST', $input['db-host'], $file);
		$file = str_replace('DB_USER', $input['db-user'], $file);
		$file = str_replace('DB_PASSWORD', $input['db-password'], $file);
		$file = str_replace('DB_NAME', $input['db-name'], $file);

		File::put(path('app').'/config/database.php', $file);

		// Migrate them up
		require_once path('sys') . 'cli' . DS . 'dependencies' . EXT;
		Laravel\CLI\Command::run(array('migrate:install'));
		Laravel\CLI\Command::run(array('migrate'));
		Laravel\CLI\Command::run(array('key:generate'));

		return Redirect::to('installer/settings')->with('good', 'Database configuration saved.');
	}

	public function get_settings()
	{
		if (! Engine::$installed OR User::count() > 0)
			return Response::error(404);

		$this->layout->nest('content', 'installer.settings');
	}

	public function post_settings()
	{
		if (! Engine::$installed OR User::count() > 0)
			return Response::error(404);

		$rules = array(
			'site-name' => 'required',
			'site-email' => 'required|email',
			'user-name' => 'required',
			'user-password' => 'required',
			'user-email' => 'required|required',
		);

		$input = Input::all();
		$validator = Validator::make($input, $rules);

		if ($validator->fails())
			return Redirect::back()
				->with('error', $validator->errors);

		Option::set('site-name', $input['site-name']);
		Option::set('site-email', $input['site-email']);

		// Create User
		$user = new User;
		$user->name = $input['user-name'];
		$user->email = $input['user-email'];
		$user->password = Hash::make($input['user-password']);
		$user->type = 'staff';
		$user->save();

		Auth::login($user->id);

		return Redirect::to('installer/finished')
			->with('error', 'Yay! You\'re done.');
	}

	public function get_finished()
	{
		$this->layout->nest('content', 'installer.finished');
	}
}

define('IS_INSTALLER', true);