<?php
class Ticket_Controller extends Base_Controller {
	public function get_index()
	{
		if (! Auth::check())
			return Response::error(404);

		Title::put('Tickets');
		$this->layout->nest('content', 'ticket.index', array(
			'tickets' => Ticket::where_user_id(Auth::user()->id)->get()
		));
	}    

	public function get_new()
	{
		Asset::add('lepture-css', 'css/editor.css');
		Asset::add('lepture-js', 'js/editor.js');

		Title::put('New Ticket');
		$this->layout->nest('content', 'ticket.new');
	}    

	public function post_new()
	{
		$rules = array(
			'subject' => 'required|min:4',
			'message' => 'required|min:10',
		);

		if (! Auth::check() ) :
			$rules['name'] = 'required';
			$rules['email'] = 'required|email|unique:users';
			$rules['password'] = 'required|min:5';
		endif;

		$input = Input::all();
		$validator = Validator::make(Input::all(), $rules);
		
		if ($validator->fails())
			return Redirect::back()
				->with_input()
				->with('error', $validator->errors);

		if (! Auth::check()) :
			// Create the user
			$user = new User;
			$user->name = $input['name'];
			$user->email = $input['email'];
			$user->password = Hash::make($input['password']);
			$user->type = 'user';
			$user->save();

			// Listen to the event...
			Auth::login($user->id);
			Event::fire('user create', array($user));
		else :
			$user = Auth::user();
		endif;

		// Save Ticket
		$ticket = new Ticket;
		$ticket->subject = $input['subject'];
		$ticket->user_id = $user->id;
		$ticket->save();

		// Save Response
		$response = new Ticket_Response;
		$response->ticket_id = $ticket->id;
		$response->user_id = $user->id;
		$response->contents = $input['message'];
		$response->save();

		// Listen-able events
		Event::fire('ticket create', $ticket->id);
		Event::fire('ticket update', $ticket->id);

		return Redirect::to('ticket/'.$ticket->id)
			->with('good', 'Ticket created.');

	}

	public function get_view($ticket_id, $arguments = array())
	{
		if (! is_numeric($ticket_id) OR ! Auth::check())
			return Response::error(404);

		$ticket = Ticket::find($ticket_id);

		if ($ticket == NULL)
			return Response::error(404);

		if ((int) $ticket->user_id !== (int) Auth::user()->id)
			return Response::error(404);

		Title::put('Ticket #'.$ticket->id);
		

		Asset::add('lepture-css', 'css/editor.css');
		Asset::add('lepture-js', 'js/editor.js');
		
		$this->layout->nest('content', 'ticket.view', array(
			'ticket' => $ticket,
			'responses' => $ticket->responses()->order_by('created_at', 'asc')->get(),
		));
	}

	public function post_view($ticket_id)
	{
		if (! is_numeric($ticket_id) OR ! Auth::check())
			return Response::error(404);

		$ticket = Ticket::find($ticket_id);

		if ($ticket == NULL)
			return Response::error(404);

		if ((int) $ticket->user_id !== (int) Auth::user()->id OR $ticket->status !== 'open')
			return Response::error(404);

		$rules = array(
			'message' => 'required|min:10',
		);

		$input = Input::all();
		$validator = Validator::make(Input::all(), $rules);
		
		if ($validator->fails())
			return Redirect::back()
				->with_input()
				->with('error', $validator->errors);

		$resp = new Ticket_Response;
		$resp->ticket_id = $ticket->id;
		$resp->user_id = Auth::user()->id;
		$resp->contents = $input['message'];
		$resp->save();

		// Update the ticket itself
		$carbon = new Carbon\Carbon;

		$ticket->updated_at = $carbon::now()->toDateTimeString();
		$ticket->save();

		return Redirect::to('ticket/'.$ticket->id)
			->with('good', 'Ticket updated.');
	}
}