<?php
class Option extends Eloquent 
{
	private static $_index = array();
	
	/**
	 * Retrieve the contents of a key
	 * 
	 * @param string Option Key
	 * @param mixed Default return value if key doesn't exist
	 * @param int|bool Should we cache the result? If so, specify a TTL
	 * @return mixed Option value or the default value
	 */
	public static function retrieve($key, $default = NULL, $cache = 5)
	{
		if (isset(static::$_index[$key]))
			return static::$_index[$key];

		// Check the cache
		if ($cache AND Cache::has('option:'.$key))
			return Cache::get('option:'.$key);

		$find = self::where_name($key)->first();

		$value = ($find == NULL) ? $default : $find->value;
		
		if ($find !== NULL) self::$_index[$key] = $find->value;

		if ($cache !== FALSE)
			Cache::put('option:'.$key, $value, $cache);

		return $value;
	}

	/**
	 * See if a key exists
	 * 
	 * @param string
	 * @return bool
	 */
	public static function exists($key)
	{
		$call = static::retrieve($key, 'NON-EXIST', FALSE);

		return ($call == 'NON-EXIST') ? FALSE : TRUE;
	}

	/**
	 * Set the Contents of a Key
	 * 
	 * @param string
	 * @param mixed Objects and arrays will be serialized
	 * @return object
	 */
	public static function set($key, $value)
	{
		if (! static::exists($key)) :
			$obj = new self;
			$obj->name = $key;
		else :
			$obj = static::where_name($key)->first();
		endif;

		$obj->value = (is_array($value) OR is_object($value)) ? serialize($value) : $value;
		$obj->save();

		Cache::forget('option:'.$key);
		return $obj;
	}
}