 <?php
/**
 * The Users Model
 *
 * Using Eloquent, it interfaces with the users table
 *
 * @access public
 */
class User extends Eloquent {
	public static $table = 'users';
	public static $timestamps = true;
	private $_is_owner = NULL;

	/**
	 * Get a raw query of the roles a user has
	 *
	 * @access public
	 * @return object
	 */
	public function roles()
	{
		return $this->has_many_and_belongs_to('Role', 'user_roles');
	}

	public function has_permission($for)
	{
		if (Cache::has('permissions:'.$this->id)) :
			$index = Cache::get('permissions:'.$this->id);
		else :
			$roles = $this->roles()->get();

			$index = array();
			if (count($roles) > 0) : foreach($roles as $d) :
				$p = json_decode($d->permissions, TRUE);

				if (count($p) == 0) continue;

				foreach($p as $k => $v) :
					$v = (bool) $v;

					if ($v)
						$index[$k] = TRUE;
				endforeach;
			endforeach; endif;

			Cache::put('permissions:'.$this->id, $index, 60);
		endif;

		if (isset($index[$for]))
			return $index[$for];
		else
			return FALSE;
	}

	public function add_role($role)
	{
		// Find the role ID
		$find = Role::where_name($role)->first();
		if ($find == NULL) return FALSE;

		if ($this->has_role($role)) return TRUE;

		$o = new User_Role;
		$o->user_id = $this->id;
		$o->role_id = $find->id;

		return $o->save();
	}

	public function remove_role($role)
	{
		return Role::where_name($role)->first()->delete();

		// Find the role ID
		$find = Role::where_name($role)->first();
		if ($find == NULL) return TRUE;

		if (! $this->has_role($role)) return TRUE;

		 return Role::find($find->id)->delete();
	}

	public function has_role($role)
	{
		$roles = $this->roles()->get();
		
		if (count($roles) == 0)
			return FALSE;

		foreach($roles as $d) :
			if ($d->name == $role)
				return TRUE;
		endforeach;

		return FALSE;
	}

	/**
	 * Get a raw query of the groups a user is apart of
	 *
	 * @access public
	 * @return object
	 */
	public function groups()
	{
		return Group_Member::where_user_id($this->attributes['id']);
	}

	/**
	 * Groups Owned
	 *
	 * @return object
	 */
	public function groups_owned()
	{
		return Group::where_owner($this->id);
	}

	/**
	 * Retrieve the User Meta
	 */
	public function meta()
	{
		return $this->has_many('Usermeta');
	}
}